@@ -7,12 +7,12 @@ import datetime
 import hashlib
 import json
 import os
-import urllib.parse
-
-from http.client import HTTPConnection, HTTPSConnection
+import urllib.parse as url_parse
+import urllib.request as url_request
 
 from mozphab import environment
 
+from .environment import USER_AGENT
 from .exceptions import (
     CommandError,
     Error,
@@ -22,6 +22,7 @@ from .exceptions import (
 from .helpers import (
     get_arcrc_path,
     read_json_field,
+    revision_title_from_commit,
     strip_differential_revision,
 )
 from .logger import logger
@@ -42,6 +43,9 @@ def normalise_reviewer(reviewer, strip_group=True):
 class ConduitAPIError(Error):
     """Raised when the Phabricator Conduit API returns an error response."""
 
+    def __init__(self, msg=None):
+        super().__init__(f"Phabricator Error: {msg if msg else 'Unknown Error'}")
+
 
 class ConduitAPI:
     def __init__(self):
@@ -60,11 +64,16 @@ class ConduitAPI:
         Returns:
             API Token string
         """
+
+        if "api_token" in cache:
+            return cache.get("api_token")
+
         token = read_json_field(
             [get_arcrc_path()], ["hosts", self.repo.api_url, "token"]
         )
         if not token:
             raise ConduitAPIError(environment.INSTALL_CERT_MSG)
+        cache.set("api_token", token)
         return token
 
     def save_api_token(self, token):
@@ -87,12 +96,13 @@ class ConduitAPI:
         if created:
             os.chmod(filename, 0o600)
 
-    def call(self, api_method, api_call_args):
+    def call(self, api_method, api_call_args, *, api_token=None):
         """Call Conduit API and return the JSON API call result.
 
         Args:
             api_method: The API method name to call, like 'differential.revision.edit'.
             api_call_args: JSON dict of call args to send.
+            api_token: Use specific token to authentication instead of saved value.
 
         Returns:
             JSON API call result object
@@ -100,37 +110,39 @@ class ConduitAPI:
         Raises:
             ConduitAPIError if the API threw an error back at us.
         """
-        url = urllib.parse.urlparse(urllib.parse.urljoin(self.repo.api_url, api_method))
-        logger.debug("%s %s", url.geturl(), api_call_args)
-
-        api_call_args = api_call_args.copy()
-        api_call_args["__conduit__"] = {"token": self.load_api_token()}
-        body = urllib.parse.urlencode(
-            {
-                "params": json.dumps(api_call_args),
-                "output": "json",
-                "__conduit__": True,
-            }
+        req_args = self._build_request(
+            method=api_method,
+            args=api_call_args,
+            token=api_token,
         )
-        # Send the POST request
-        if url.scheme == "https":
-            conn = HTTPSConnection(url.netloc)
-        elif environment.HTTP_ALLOWED:
-            # Allow for an HTTP connection in suite.
-            conn = HTTPConnection(url.netloc)
-        else:
-            raise CommandError("Only https connections are allowed.")
+        logger.debug("%s %s", req_args["url"], api_call_args)
 
-        conn.request("POST", url.geturl(), body=body)
+        with url_request.urlopen(url_request.Request(**req_args)) as r:
+            res = json.load(r)
+        if res["error_code"]:
+            raise ConduitAPIError(res.get("error_info", "Error %s" % res["error_code"]))
+        return res["result"]
 
-        # Read the response as JSON
-        response = json.loads(conn.getresponse().read().decode("utf-8"))
-        if response["error_code"]:
-            raise ConduitAPIError(
-                response.get("error_info", "Error %s" % response["error_code"])
-            )
-
-        return response["result"]
+    def _build_request(self, *, method, args, token):
+        """Return dict with Request args for calling the specified conduit method."""
+        return dict(
+            url=url_parse.urljoin(self.repo.api_url, method),
+            method="POST",
+            headers={"User-Agent": USER_AGENT},
+            data=url_parse.urlencode(
+                {
+                    "params": json.dumps(
+                        {
+                            **args,
+                            "__conduit__": {"token": token or self.load_api_token()},
+                        },
+                        separators=(",", ":"),
+                    ),
+                    "output": "json",
+                    "__conduit__": True,
+                }
+            ).encode(),
+        )
 
     def ping(self):
         """Sends a ping to the Phabricator server using `conduit.ping` API.
@@ -453,19 +465,16 @@ class ConduitAPI:
     def create_revision(
         self,
         commit,
-        title,
         summary,
         diff_phid,
-        has_commit_reviewers,
-        wip=False,
         check_in_needed=False,
     ):
         """Create a new revision in Phabricator."""
         transactions = [
-            dict(type="title", value=title),
+            dict(type="title", value=revision_title_from_commit(commit)),
             dict(type="summary", value=summary),
         ]
-        if has_commit_reviewers and not wip:
+        if commit["has-reviewers"] and not commit["wip"]:
             self.update_revision_reviewers(transactions, commit)
 
         if commit["bug-id"]:
@@ -473,24 +482,22 @@ class ConduitAPI:
         return self.edit_revision(
             transactions=transactions,
             diff_phid=diff_phid,
-            wip=wip,
+            wip=commit["wip"],
             check_in_needed=check_in_needed,
         )
 
     def update_revision(
         self,
         commit,
-        has_commit_reviewers,
         existing_reviewers,
         diff_phid=None,
-        wip=False,
         comment=None,
         check_in_needed=False,
     ):
         """Update an existing revision in Phabricator."""
         # Update the title and summary
         transactions = [
-            dict(type="title", value=commit["title"]),
+            dict(type="title", value=revision_title_from_commit(commit)),
             dict(type="summary", value=strip_differential_revision(commit["body"])),
         ]
 
@@ -499,7 +506,7 @@ class ConduitAPI:
             transactions.append(dict(type="comment", value=comment))
 
         # Add reviewers only if revision lacks them
-        if has_commit_reviewers and not wip:
+        if commit["has-reviewers"] and not commit["wip"]:
             if not existing_reviewers:
                 self.update_revision_reviewers(transactions, commit)
 
@@ -515,7 +522,7 @@ class ConduitAPI:
             transactions=transactions,
             diff_phid=diff_phid,
             rev_id=commit["rev-id"],
-            wip=wip,
+            wip=commit["wip"],
             check_in_needed=check_in_needed,
         )
 
@@ -525,29 +532,46 @@ class ConduitAPI:
         diff_phid=None,
         rev_id=None,
         wip=False,
-        force_wip=False,
         check_in_needed=False,
     ):
         """Edit (create or update) a revision."""
         trans = list(transactions or [])
+        set_wip_later = False
+
         # diff_phid is not present for changes in revision settings (like WIP)
         if diff_phid:
             trans.append(dict(type="update", value=diff_phid))
 
-        set_wip_later = False
+        existing_status = None
+        if rev_id:
+            try:
+                args = dict(ids=[int(rev_id)])
+            except ValueError:
+                args = dict(phids=[rev_id])
+            existing_revision = conduit.get_revisions(**args)[0]
+            existing_status = existing_revision["fields"]["status"]["value"]
+
+        # Set revision for changes-planned or needs-review as required.
+        # Phabricator will throw an error if we attempt to set a status to the same
+        # as the current status.
         if wip:
-            if rev_id and not force_wip:
-                # Set "changes planned" in a new request called after the update one.
-                # Phab API validation would return with an error if "changes planned
-                # would be set in the first API call.
-                existing_revision = conduit.get_revisions(ids=[int(rev_id)])[0]
-                set_wip_later = (
-                    existing_revision["fields"]["status"]["value"] == "changes-planned"
-                )
+            # Phabriactor will automatically set the revision to needs-review
+            # after the call to differential.revision.edit.  Flag that we'll need to
+            # make a subsequent API call to set the status to changes-planned to match
+            # our WIP state.
+            if existing_status == "changes-planned":
+                set_wip_later = True
+            else:
+                trans.append(dict(type="plan-changes", value=True))
 
-        if force_wip or wip and not set_wip_later:
-            trans.append(dict(type="plan-changes", value=True))
+        # Let the Phabricator sort out the correct status for new revisions to avoid
+        # emails being sent before phabbugs has processed the revision.
+        # Updates to existing reviews that have been accepted shouldn't be triggered
+        # for re-review.
+        elif existing_status and existing_status not in ("needs-review", "accepted"):
+            trans.append(dict(type="request-review", value=True))
 
+        # Add the check-in-needed tag if required.
         if check_in_needed:
             logger.info(f"Adding #{CHECK_IN_NEEDED} to revision.")
             check_in_needed_phid = conduit.get_project_phid(CHECK_IN_NEEDED)
@@ -559,17 +583,24 @@ class ConduitAPI:
                     f"Project #{CHECK_IN_NEEDED} doesn't exist."
                 )
 
+        # Call differential.revision.edit
         api_call_args = dict(transactions=trans)
-
         if rev_id:
             api_call_args["objectIdentifier"] = rev_id
-
         revision = self.call("differential.revision.edit", api_call_args)
         if not revision:
             raise ConduitAPIError("Can't edit the revision.")
 
-        if wip and set_wip_later:
-            return self.edit_revision(rev_id=rev_id, force_wip=True)
+        # Set changes-planned if required.
+        # Note set_wip_later can only be true if there's an existing revision.
+        if set_wip_later:
+            revision = self.call(
+                "differential.revision.edit",
+                dict(
+                    objectIdentifier=rev_id,
+                    transactions=[dict(type="plan-changes", value=True)],
+                ),
+            )
 
         return revision
 
@@ -614,7 +645,7 @@ class ConduitAPI:
                 "author": commit["author-name"],
                 "authorEmail": commit["author-email"],
                 "time": commit["author-date-epoch"],
-                "summary": commit["title-preview"],
+                "summary": revision_title_from_commit(commit),
                 "message": message,
                 "commit": conduit.repo.get_public_node(commit["node"]),
                 "parents": [conduit.repo.get_public_node(commit["parent"])],
@@ -670,11 +701,11 @@ class ConduitAPI:
 
         return file_phid
 
-    def whoami(self):
+    def whoami(self, *, api_token=None):
         if "whoami" in cache:
             return cache.get("whoami")
 
-        who = self.call("user.whoami", {})
+        who = self.call("user.whoami", {}, api_token=api_token)
         cache.set("whoami", who)
         return who
 
