@@ -4,6 +4,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import calendar
+import contextlib
 import logging
 import logging.handlers
 import os
@@ -22,19 +23,21 @@ LOG_MAX_SIZE = 1024 * 1024 * 50
 LOG_BACKUPS = 5
 
 
+_handlers = list()
+
+
 class ColourFormatter(logging.Formatter):
-    def __init__(self, debug, has_ansi):
-        if debug:
+    def __init__(self):
+        if environment.DEBUG:
             fmt = "%(levelname)-8s %(asctime)-13s %(message)s"
         else:
             fmt = "%(message)s"
         super().__init__(fmt)
         self.log_colours = {"WARNING": 34, "ERROR": 31}  # blue, red
-        self.has_ansi = has_ansi
 
     def format(self, record):
         result = super().format(record)
-        if self.has_ansi and record.levelname in self.log_colours:
+        if environment.HAS_ANSI and record.levelname in self.log_colours:
             result = "\033[%sm%s\033[0m" % (self.log_colours[record.levelname], result)
         return result
 
@@ -42,24 +45,24 @@ class ColourFormatter(logging.Formatter):
 def init_logging():
     """Initialize logging."""
     log_file = os.path.join(environment.MOZBUILD_PATH, "moz-phab.log")
-    stdout_handler = logging.StreamHandler(sys.stdout)
-    stdout_handler.setFormatter(
-        ColourFormatter(environment.DEBUG, environment.HAS_ANSI)
-    )
-    stdout_handler.setLevel(logging.DEBUG if environment.DEBUG else logging.INFO)
-    logger.addHandler(stdout_handler)
+    handler = logging.StreamHandler(sys.stdout)
+    handler.setFormatter(ColourFormatter())
+    handler.setLevel(logging.DEBUG if environment.DEBUG else logging.INFO)
+    logger.addHandler(handler)
+    _handlers.append(handler)
 
-    file_handler = logging.handlers.RotatingFileHandler(
+    handler = logging.handlers.RotatingFileHandler(
         filename=log_file,
         maxBytes=LOG_MAX_SIZE,
         backupCount=LOG_BACKUPS,
         encoding="utf8",
     )
-    file_handler.setFormatter(
+    handler.setFormatter(
         logging.Formatter("%(asctime)-13s %(levelname)-8s %(message)s")
     )
-    file_handler.setLevel(logging.DEBUG)
-    logger.addHandler(file_handler)
+    handler.setLevel(logging.DEBUG)
+    logger.addHandler(handler)
+    _handlers.append(handler)
 
     logger.setLevel(logging.DEBUG)
 
@@ -74,4 +77,14 @@ def init_logging():
         )
         if (now - file_time) / (60 * 60 * 24) > 8:
             logger.debug("deleting old log file: %s" % os.path.basename(filename))
-            os.unlink(filename)
+            with contextlib.suppress(IOError):
+                os.unlink(filename)
+
+
+def stop_logging():
+    """Remove our logging handlers and close files."""
+
+    for handler in _handlers:
+        logger.removeHandler(handler)
+        if isinstance(handler, logging.FileHandler):
+            handler.close()
