@@ -8,7 +8,6 @@ import subprocess
 import sys
 import time
 import urllib.request
-import __main__ as script_module
 
 from distutils.dist import Distribution
 from pathlib import Path
@@ -16,14 +15,12 @@ from pkg_resources import parse_version
 
 from mozphab import environment
 
-from .arcanist import update_arc
 from .config import config
 from .environment import MOZPHAB_VERSION
 from .exceptions import Error
 from .logger import logger, stop_logging
 from .subprocess_wrapper import check_call
 
-ARC_UPDATE_FREQUENCY = 24 * 7  # hours
 SELF_UPDATE_FREQUENCY = 24 * 3  # hours
 
 
@@ -34,16 +31,8 @@ def get_pypi_info():
     return response["info"]
 
 
-def check_for_updates(with_arc=True):
+def check_for_updates():
     """Log a message if an update is required/available"""
-    # Update arc.
-    if (
-        with_arc
-        and config.arc_last_check >= 0
-        and time.time() - config.arc_last_check > ARC_UPDATE_FREQUENCY * 60 * 60
-    ):
-        update_arc()
-
     # Update self.
     if (
         config.self_last_check >= 0
@@ -112,6 +101,14 @@ def self_upgrade():
         + ["--disable-pip-version-check"]
     )
 
+    if config.get_pre_releases:
+        command += ["--pre"]
+
+    # sys.path[0] is the directory containing the script that was used to
+    # start python. This will be something like:
+    # "<python environment>/bin" or "<python environment>\Scripts" (Windows)
+    script_dir = Path(sys.path[0])
+
     # If moz-phab was installed with --user, we need to pass it to pip
     # Create "install" distutils command with --user to find the scripts_path
     d = Distribution()
@@ -122,7 +119,6 @@ def self_upgrade():
     i.prefix = i.exec_prefix = i.home = i.install_base = i.install_platbase = None
     i.finalize_options()
     # Checking if the moz-phab script is installed in user's scripts directory
-    script_dir = Path(script_module.__file__).resolve().parent
     user_dir = Path(i.install_scripts).resolve()
     if script_dir == user_dir:
         command.append("--user")
